==> prompt_injection_bench-0.1.7.tar.gz: prompt_injection_bench-0.1.7/prompt_injection_bench/server.py <==
import asyncio
import json
import os
import re
from asyncio import tasks

from datasets import load_dataset
from dotenv import load_dotenv
from fastapi import FastAPI, websockets
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastapi import WebSocket

from prompt_injection_bench.gemini_async_prompt_guard import GeminiAsyncPromptGuard
from prompt_injection_bench.openai_async_prompt_guard import OpenAIAsyncPromptGuard
from pydantic import BaseModel

load_dotenv()


class PromptCheckRequest(BaseModel):
    text: str
    extractedUrls: list[str]  # Add this line to include the list of URLs


class URLAddRequest(BaseModel):
    url: str


class LLMState(BaseModel, arbitrary_types_allowed=True):
    pass


class OpenAIState(LLMState, arbitrary_types_allowed=True):
    prompt_guard: OpenAIAsyncPromptGuard
    injection_attack: int
    not_injection_attack: int
    missed_attack: int


class GeminiState(LLMState, arbitrary_types_allowed=True):
    prompt_guard: GeminiAsyncPromptGuard
    injection_attack: int
    not_injection_attack: int
    missed_attack: int


dataset = load_dataset("jackhhao/jailbreak-classification")


def mount_static_directory(app: FastAPI):
    base_dir = os.path.dirname(os.path.abspath(__file__))
    static_dir = os.path.join(base_dir, 'static')
    app.mount("/static", StaticFiles(directory=static_dir), name="static")


# Access the "test" split of the dataset
test_data = dataset['test']

prompt_bench_app = FastAPI()
# Call the function to mount the static directory
mount_static_directory(prompt_bench_app)


def process_return_value(return_value, state: LLMState):
    # Check if the return value is an integer, which indicates a status code was returned
    status_code = return_value[0]
    response = return_value[1].lower()

    # Process based on the status code
    if status_code == 200:
        pass
    elif status_code == 401:
        print("Authentication Error. Please check your credentials.")
        state.prompt_guard.enabled = False
    elif status_code == 429:
        print("Rate Limit Exceeded. Please try again later.")
    elif 400 <= status_code < 500:
        print("Client Error. Status code:", return_value)
    elif 500 <= status_code < 600:
        print("Server Error. Status code:", return_value)
    else:
        print("Received an unexpected status code:", return_value)
        raise ValueError("Received an unexpected status code:", return_value)
    # Check if the return value is a string, which indicates a successful operation

    pattern_true = r'\btrue\b\.?'
    pattern_false= r'\bfalse\b\.?'

    matches = re.findall(pattern_true, response)
    if len(matches) > 0:
        state.injection_attack += 1
    elif len(re.findall(pattern_false, response)) > 0:
        state.not_injection_attack += 1
    else:
        state.missed_attack += 1


async def generate_response_with_timeout(prompt_guard, prompt, timeout):
    try:
        return await asyncio.wait_for(prompt_guard.generate_response(prompt), timeout=timeout)
    except asyncio.TimeoutError:
        return None


@prompt_bench_app.on_event("startup")
async def startup_event():
    openai_prompt_guard = OpenAIAsyncPromptGuard()
    prompt_bench_app.state.openai = OpenAIState(enabled=True, prompt_guard=openai_prompt_guard,
                                                injection_attack=0, not_injection_attack=0, missed_attack=0)
    if openai_prompt_guard.enabled is False:
        print("No OpenAI API key found, OpenAI prompt injection detection is disabled")

    gemini_prompt_guard = GeminiAsyncPromptGuard()
    prompt_bench_app.state.gemini = GeminiState(enabled=True, prompt_guard=gemini_prompt_guard,
                                                injection_attack=0, not_injection_attack=0, missed_attack=0)
    if gemini_prompt_guard.enabled is False:
        print("No Google API key found, gemini prompt injection detection is disabled")


@prompt_bench_app.get("/healthz")
def health_check():
    return {"status": "ok"}


# Serve HTML file from the root route
@prompt_bench_app.get("/", response_class=HTMLResponse)
async def read_root():
    base_dir = os.path.dirname(os.path.abspath(__file__))
    index_file_path = os.path.join(base_dir, "index.html")
    with open(index_file_path, "r") as file:
        return HTMLResponse(content=file.read())


# Create a WebSocket route for starting analysis and streaming results
@prompt_bench_app.websocket("/ws/start-analysis")
async def start_analysis(websocket: WebSocket):
    await websocket.accept()

    timeout = 20
    total_generated_texts = 0
    openai_prompt_guard = prompt_bench_app.state.openai.prompt_guard
    gemini_prompt_guard = prompt_bench_app.state.gemini.prompt_guard

    for prompt, prompt_type in zip(test_data['prompt'], test_data['type']):

        llm_tasks = []

        if prompt_type == "benign":
            continue

        # Increment the total generated texts counter
        total_generated_texts += 1

        if openai_prompt_guard.enabled:
            # Interacting with openai_prompt_guard
            llm_tasks.append(generate_response_with_timeout(openai_prompt_guard, prompt, timeout))
            # openai_generated_text = await openai_prompt_guard.generate_response(prompt)
            # Check and count injection attacks for openai_prompt_guard

        if gemini_prompt_guard.enabled:
            # Interacting with gemini_prompt_guard
            llm_tasks.append(generate_response_with_timeout(gemini_prompt_guard, prompt, timeout))
            # gemini_generated_text = await gemini_prompt_guard.generate_response(prompt)

        task_results = await asyncio.gather(*llm_tasks)
        if task_results[0] is not None:
            process_return_value(task_results[0], prompt_bench_app.state.openai)
        if task_results[1] is not None:
            process_return_value(task_results[1], prompt_bench_app.state.gemini)

        # Print the tabulated results for each LLM if they are enabled
        # print(f"Total Generated Prompts: {total_generated_texts}")
        # if openai_prompt_guard_enabled:
        #     print(f"OpenAI Injection Attack Detected: {prompt_bench_app.state.openai.injection_attack_detected}")
        # if gemini_prompt_guard_enabled:
        #     print(f"Gemini Injection Attack Detected: {prompt_bench_app.state.gemini.injection_attack_detected}")
        # print("\n" + "=" * 40 + "\n")
        results = {"Total Injection Prompts": total_generated_texts,
                   "OpenAI Detected": prompt_bench_app.state.openai.injection_attack,
                   "Gemini Detected": prompt_bench_app.state.gemini.injection_attack}

        try:
            await websocket.send_text(json.dumps(results))
        except websockets.exceptions.ConnectionClosedError as e:
            print("Connection closed")
            break

    print("Analysis complete")

==> prompt_injection_bench-0.1.8.tar.gz: prompt_injection_bench-0.1.8/prompt_injection_bench/server.py <==
import asyncio
import json
import os
import re
from asyncio import tasks

from datasets import load_dataset
from dotenv import load_dotenv
from fastapi import FastAPI, websockets
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastapi import WebSocket

from prompt_injection_bench.azure_openai_async_prompt_guard import AzureOpenAIAsyncPromptGuard
from prompt_injection_bench.gemini_async_prompt_guard import GeminiAsyncPromptGuard
from prompt_injection_bench.openai_async_prompt_guard import OpenAIAsyncPromptGuard
from pydantic import BaseModel

load_dotenv()


class PromptCheckRequest(BaseModel):
    text: str
    extractedUrls: list[str]  # Add this line to include the list of URLs


class URLAddRequest(BaseModel):
    url: str


class LLMState(BaseModel, arbitrary_types_allowed=True):
    pass


class OpenAIState(LLMState, arbitrary_types_allowed=True):
    prompt_guard: OpenAIAsyncPromptGuard
    injection_attack: int
    not_injection_attack: int
    missed_attack: int


class GeminiState(LLMState, arbitrary_types_allowed=True):
    prompt_guard: GeminiAsyncPromptGuard
    injection_attack: int
    not_injection_attack: int
    missed_attack: int


class AzureJailbreakState(LLMState, arbitrary_types_allowed=True):
    prompt_guard: AzureOpenAIAsyncPromptGuard
    injection_attack: int
    not_injection_attack: int
    missed_attack: int


dataset = load_dataset("jackhhao/jailbreak-classification")


def mount_static_directory(app: FastAPI):
    base_dir = os.path.dirname(os.path.abspath(__file__))
    static_dir = os.path.join(base_dir, 'static')
    app.mount("/static", StaticFiles(directory=static_dir), name="static")


# Access the "test" split of the dataset
test_data = dataset['test']

prompt_bench_app = FastAPI()
# Call the function to mount the static directory
mount_static_directory(prompt_bench_app)


def process_return_value(return_value, state: LLMState):
    # Check if the return value is an integer, which indicates a status code was returned
    status_code = return_value[0]
    response = return_value[1].lower()

    # Process based on the status code
    if status_code == 200:
        pass
    elif status_code == 401:
        print("Authentication Error. Please check your credentials.")
        state.prompt_guard.enabled = False
    elif status_code == 429:
        print("Rate Limit Exceeded. Please try again later.")
    elif 400 <= status_code < 500:
        print("Client Error. Status code:", return_value)
    elif 500 <= status_code < 600:
        print("Server Error. Status code:", return_value)
    else:
        print("Received an unexpected status code:", return_value)
        raise ValueError("Received an unexpected status code:", return_value)
    # Check if the return value is a string, which indicates a successful operation

    pattern_true = r'\btrue\b\.?'
    pattern_false = r'\bfalse\b\.?'

    matches = re.findall(pattern_true, response)
    if len(matches) > 0:
        state.injection_attack += 1
    elif len(re.findall(pattern_false, response)) > 0:
        state.not_injection_attack += 1
    else:
        state.missed_attack += 1


async def generate_response_with_timeout(prompt_guard, prompt, timeout):
    try:
        return await asyncio.wait_for(prompt_guard.generate_response(prompt), timeout=timeout)
    except asyncio.TimeoutError:
        return None


@prompt_bench_app.on_event("startup")
async def startup_event():
    openai_prompt_guard = OpenAIAsyncPromptGuard()
    prompt_bench_app.state.openai = OpenAIState(enabled=True, prompt_guard=openai_prompt_guard,
                                                injection_attack=0, not_injection_attack=0, missed_attack=0)
    if openai_prompt_guard.enabled is False:
        print("No OpenAI API key found, OpenAI prompt injection detection is disabled")

    gemini_prompt_guard = GeminiAsyncPromptGuard()
    prompt_bench_app.state.gemini = GeminiState(enabled=True, prompt_guard=gemini_prompt_guard,
                                                injection_attack=0, not_injection_attack=0, missed_attack=0)
    if gemini_prompt_guard.enabled is False:
        print("No Google API key found, gemini prompt injection detection is disabled")

    azure_jailbreak_prompt_guard = AzureOpenAIAsyncPromptGuard()
    prompt_bench_app.state.azure_jailbreak = AzureJailbreakState(enabled=True, prompt_guard=azure_jailbreak_prompt_guard,
                                                                 injection_attack=0, not_injection_attack=0,
                                                                 missed_attack=0)
    if azure_jailbreak_prompt_guard.enabled is False:
        print("No OpenAI API key found, OpenAI prompt injection detection is disabled")


@prompt_bench_app.get("/healthz")
def health_check():
    return {"status": "ok"}


# Serve HTML file from the root route
@prompt_bench_app.get("/", response_class=HTMLResponse)
async def read_root():
    base_dir = os.path.dirname(os.path.abspath(__file__))
    index_file_path = os.path.join(base_dir, "index.html")
    with open(index_file_path, "r") as file:
        return HTMLResponse(content=file.read())


# Create a WebSocket route for starting analysis and streaming results
@prompt_bench_app.websocket("/ws/start-analysis")
async def start_analysis(websocket: WebSocket):
    await websocket.accept()

    timeout = 20
    total_generated_texts = 0
    openai_prompt_guard = prompt_bench_app.state.openai.prompt_guard
    gemini_prompt_guard = prompt_bench_app.state.gemini.prompt_guard
    azure_jailbreak_prompt_guard = prompt_bench_app.state.azure_jailbreak.prompt_guard

    for prompt, prompt_type in zip(test_data['prompt'], test_data['type']):

        llm_tasks = []

        if prompt_type == "benign":
            continue

        # Increment the total generated texts counter
        total_generated_texts += 1

        if openai_prompt_guard.enabled:
            # Interacting with openai_prompt_guard
            llm_tasks.append(generate_response_with_timeout(openai_prompt_guard, prompt, timeout))

        if gemini_prompt_guard.enabled:
            # Interacting with gemini_prompt_guard
            llm_tasks.append(generate_response_with_timeout(gemini_prompt_guard, prompt, timeout))

        if azure_jailbreak_prompt_guard.enabled:
            # Interacting with gemini_prompt_guard
            llm_tasks.append(generate_response_with_timeout(azure_jailbreak_prompt_guard, prompt, timeout))

        # Tasks can return None for a few reasons, and we need to catch it
        # https://docs.python.org/3/library/asyncio.html
        task_results = await asyncio.gather(*llm_tasks)
        if task_results[0] is not None:
            process_return_value(task_results[0], prompt_bench_app.state.openai)
        if task_results[1] is not None:
            process_return_value(task_results[1], prompt_bench_app.state.gemini)
        if task_results[2] is not None:
            process_return_value(task_results[2], prompt_bench_app.state.azure_jailbreak)

        # Uncomment the following lines to print the tabulated results for each LLM if they are enabled
        # Print the tabulated results for each LLM if they are enabled
        # print(f"Total Generated Prompts: {total_generated_texts}")
        # if openai_prompt_guard_enabled:
        #     print(f"OpenAI Injection Attack Detected: {prompt_bench_app.state.openai.injection_attack_detected}")
        # if gemini_prompt_guard_enabled:
        #     print(f"Gemini Injection Attack Detected: {prompt_bench_app.state.gemini.injection_attack_detected}")
        # print("\n" + "=" * 40 + "\n")
        results = {"Total Injection Prompts": total_generated_texts,
                   "OpenAI Detected": prompt_bench_app.state.openai.injection_attack,
                   "Gemini Detected": prompt_bench_app.state.gemini.injection_attack,
                   "Azure OpenAI Jailbreak": prompt_bench_app.state.azure_jailbreak.injection_attack}

        try:
            await websocket.send_text(json.dumps(results))
        except websockets.exceptions.ConnectionClosedError as e:
            print("Connection closed")
            break

    print("Analysis complete")
